@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2007 G.P. Halkes
+/* Copyright (C) 2006-2011 G.P. Halkes
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3, as
    published by the Free Software Foundation.
@@ -15,9 +15,17 @@
 #ifndef DEFINITIONS_H
 #define DEFINITIONS_H
 #include <stdio.h>
+#include <limits.h>
+
+#if defined(USE_GETTEXT) || defined(USE_UNICODE)
+#	include <locale.h>
+#endif
+
+#if defined(USE_UNICODE) && defined(USE_NL_LANGINFO)
+#	include <langinfo.h>
+#endif
 
 #ifdef USE_GETTEXT
-#	include <locale.h> 
 #	include <libintl.h>
 #	define _(String) gettext(String)
 #else
@@ -25,15 +33,15 @@
 #endif
 #define N_(String) String
 
-#ifdef __GNUC__
-void fatal(const char *fmt, ...) __attribute__((format (printf, 1, 2))) __attribute__((noreturn));
+#ifdef USE_UNICODE
+#define ONLY_UNICODE(_x) _x
+#define SWITCH_UNICODE(a, b) a
 #else
-/*@noreturn@*/void fatal(const char *fmt, ...);
+#define ONLY_UNICODE(_x)
+#define SWITCH_UNICODE(a, b) b
 #endif
 
-#define PANIC() fatal(_("Program logic error at %s:%d\n"), __FILE__, __LINE__)
-#define ASSERT(_condition) do { if (!(_condition)) PANIC(); } while(0)
-
+/*==== Misc definitions ====*/
 /* Define a bool type if not already defined (C++ and C99 do)*/
 #if !(defined(__cplusplus) || (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 19990601L))
 /*@-incondefs@*/
@@ -43,39 +51,38 @@ typedef enum {false, true} bool;
 #include <stdbool.h>
 #endif
 
-#define TEMPLATE "/tmp/dwdiffXXXXXXXX"
-#define TEMPLATE_LENGTH sizeof(TEMPLATE)
+/*==== Configuration definitions ====*/
+#ifndef NO_STRDUP
+#define strdupA strdup
+#endif
 
-typedef struct {
-	FILE *file;
-	char name[TEMPLATE_LENGTH];
-} TempFile;
+#define VERSION_STRING "2.0.1"
+
+typedef struct CharData CharData;
+
+#include "vector.h"
+#include "stream.h"
+#include "tempfile.h"
+#include "unicode.h"
+#include "buffer.h"
+#include "diff/diff.h"
+
+typedef lin ValueType;
+#define VALUE_MAX LIN_MAX
+
+typedef VECTOR(ValueType, ValueTypeVector);
 
 typedef struct {
 	const char *name;
-	FILE *input;
+	Stream *input;
+	ValueTypeVector diffTokens;
 	TempFile *tokens;
 	TempFile *whitespace;
 	int lastPrinted;
+	CharBuffer whitespaceBuffer;
+	bool whitespaceBufferUsed;
 } InputFile;
 
-#ifndef DIFF
-#	define DIFF "diff"
-#endif
-
-#ifdef NO_MINUS_A
-#	define MINUS_A ""
-#else
-#	define MINUS_A " -a"
-#endif
-
-#define DIFF_COMMAND DIFF MINUS_A
-/* Note: for the case that NO_MINUS_I is declared,
-   this one is simply never used */
-#define DIFF_COMMAND_CI DIFF_COMMAND " -i"
-
-void doDiff(void);
-
 typedef struct {
 	int added,
 		deleted,
@@ -87,58 +94,34 @@ typedef struct {
 extern Statistics statistics;
 extern int differences;
 
-int yylex(void);
-extern FILE *yyin;
-
-
-TempFile *tempFile(void);
-
-#define DEFAULT_LINENUMBER_WIDTH 4
-#define BITMASK_SIZE 64
-struct {
-	InputFile oldFile,
-		newFile;
-	const char *delStart,
-		*delStop,
-		*addStart,
-		*addStop,
-		*delColor,
-		*addColor;
-	size_t delStartLen,
-		delStopLen,
-		addStartLen,
-		addStopLen,
-		delColorLen,
-		addColorLen;
-	char delimiters[BITMASK_SIZE],
-		whitespace[BITMASK_SIZE];
-	bool printDeleted,
-		printAdded,
-		printCommon,
-		needMarkers,
-		needStartStop;
-	bool printer,
-		less,
-		statistics,
-		ignoreCase,
-		colorMode;
-	bool transliterate;
-	int whitespaceDelimiter;
-	const char *diffOption;
-	int lineNumbers;
-	bool context;
-	int contextLines;
-} option;
-
 #define SET_BIT(x, b) do { (x)[(b)>>3] |= 1 << ((b) & 0x7); } while (0);
 #define RESET_BIT(x, b) do { (x)[(b)>>3] &= ~(1 << ((b) & 0x7)); } while (0);
 #define TEST_BIT(x, b) ((x)[(b)>>3] & (1 << ((b) & 0x7)))
 
-void parseCmdLine(int argc, char *argv[]);
-
-void initContextBuffers(void);
-void addchar(char c, bool common);
-void printLineNumbers(int oldLineNumber, int newLineNumber);
-void writeString(const char *string, size_t bytes);
+struct CharData {
+	int singleChar;
 
+#ifdef USE_UNICODE
+	struct {
+		UTF16Buffer original; /* UTF-16 encoded original input after clean-up */
+		UTF16Buffer converted; /* UTF-16 encoded string for comparison purposes. */
+		UTF16Buffer casefolded; /* UTF-16 encoded string for comparison purposes, case folded version. */
+	} UTF8Char;
+#endif
+};
+
+#ifdef USE_UNICODE
+extern bool UTF8Mode;
+#endif
+extern CharData charData;
+
+void doDiff(void);
+
+enum {
+	CAT_OTHER,
+	CAT_DELIMITER,
+	CAT_WHITESPACE
+};
+
+int classifyChar(void);
 #endif
